"""Metadata module for managing Buckets."""
# this will be autogenerated by pop-create with the resource id based on the path value from JSON schema for GET
PATH = "b/{bucket}"

NATIVE_RESOURCE_TYPE = "storage.buckets"
