async def gather(hub):
    """
    Authenticate to the gcp with it's traditional cli tool's config/environment
    """
    sub_profiles = {}

    # TODO add profiles based on traditional gcp cli tool auth

    return sub_profiles
